@@ -98,17 +98,17 @@ class intbv(object):
     # copy methods
     def __copy__(self):
         c = type(self)(self._val)
-	c._min = self._min
-	c._max = self._max
-	c._nrbits = self._nrbits
-	return c
+        c._min = self._min
+        c._max = self._max
+        c._nrbits = self._nrbits
+        return c
 
     def __deepcopy__(self, visit):
         c = type(self)(self._val)
-	c._min = self._min
-	c._max = self._max
-	c._nrbits = self._nrbits
-	return c
+        c._min = self._min
+        c._max = self._max
+        c._nrbits = self._nrbits
+        return c
 
     # iterator method
     def __iter__(self):
@@ -436,11 +436,37 @@ class intbv(object):
     def __index__(self):
         return int(self._val)
         
-    def __cmp__(self, other):
+    # comparisons
+    def __eq__(self, other):
         if isinstance(other, intbv):
-            return cmp(self._val, other._val)
+            return self._val == other._val
         else:
-            return cmp(self._val, other)
+            return self._val == other
+    def __ne__(self, other):
+        if isinstance(other, intbv):
+            return self._val != other._val
+        else:
+            return self._val != other
+    def __lt__(self, other):
+        if isinstance(other, intbv):
+            return self._val < other._val
+        else:
+            return self._val < other
+    def __le__(self, other):
+        if isinstance(other, intbv):
+            return self._val <= other._val
+        else:
+            return self._val <= other
+    def __gt__(self, other):
+        if isinstance(other, intbv):
+            return self._val > other._val
+        else:
+            return self._val > other
+    def __ge__(self, other):
+        if isinstance(other, intbv):
+            return self._val >= other._val
+        else:
+            return self._val >= other
 
     # representation 
     def __str__(self):
