@@ -1,7 +1,7 @@
-#  This file is part of the myhdl library, a Python package for using
+
 #  Python as a Hardware Description Language.
 #
-#  Copyright (C) 2003-2012 Jan Decaluwe
+#  Copyright (C) 2003-2014 Jan Decaluwe
 #
 #  The myhdl library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public License as
@@ -147,6 +147,7 @@ class _ToVHDLConvertor(object):
         ### initialize properly ###
         _genUniqueSuffix.reset()
         _enumTypeSet.clear()
+        _enumPortTypeSet.clear()
         _constDict.clear()
         _extConstDict.clear()
 
@@ -170,9 +171,11 @@ class _ToVHDLConvertor(object):
             # add enum types to port-related set
             if isinstance(s._val, EnumItemType):
                 obj = s._val._type
-                assert obj in _enumTypeSet
-                _enumTypeSet.remove(obj)
-                _enumPortTypeSet.add(obj)
+                if obj in _enumTypeSet:
+                    _enumTypeSet.remove(obj)
+                    _enumPortTypeSet.add(obj)
+                else:
+                    assert obj in _enumPortTypeSet
 
         doc = _makeDoc(inspect.getdoc(func))
         
@@ -256,6 +259,8 @@ def _writeCustomPackage(f, intf):
     print >> f
     print >> f, "package pck_%s is" % intf.name
     print >> f
+    print >> f, "attribute enum_encoding: string;"
+    print >> f
     sortedList = list(_enumPortTypeSet)
     sortedList.sort(cmp=lambda a, b: cmp(a._name, b._name))
     for t in sortedList:
@@ -414,8 +419,8 @@ def _getRangeString(s):
     elif s._nrbits is not None:
         ls = getattr(s, 'lenStr', False)
         if ls:
-	    msb = ls + '-1'
-	else:
+            msb = ls + '-1'
+        else:
             msb = s._nrbits-1 
         return "(%s downto 0)" %  msb
     else:
@@ -856,7 +861,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             if isinstance(lhs.value, ast.Name):
                 sig = self.tree.symdict[lhs.value.id]
                 if not sig._numeric:
-	            #if not isinstance(rhs, ast.Num):
+                    #if not isinstance(rhs, ast.Num):
                     convOpen, convClose = "std_logic_vector(", ")"
             self.write(' <= ')
             self.SigAss = False
@@ -901,6 +906,8 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
         fn = node.func
         # assert isinstance(fn, astNode.Name)
         f = self.getObj(fn)
+        fname = ''
+        pre, suf = '', ''
         opening, closing = '(', ')'
         sep = ", "
         if f is bool:
@@ -964,19 +971,25 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             self.write(" ns")
             return
         elif f is concat:
+            pre, suf = self.inferCast(node.vhd, node.vhdOri)
             opening, closing =  "unsigned'(", ")"
             sep = " & "
         elif hasattr(node, 'tree'):
-            self.write(node.tree.name)
+            pre, suf = self.inferCast(node.vhd, node.tree.vhd)
+            fname = node.tree.name
         else:
             self.write(f.__name__)
         if node.args:
+            self.write(pre)
+            # TODO rewrite making use of fname variable
+            self.write(fname)
             self.write(opening)
             self.visit(node.args[0])
             for arg in node.args[1:]:
                 self.write(sep)
                 self.visit(arg)
             self.write(closing)
+            self.write(suf)
         if hasattr(node, 'tree'):
             if node.tree.kind == _kind.TASK:
                 Visitor = _ConvertTaskVisitor
@@ -1193,7 +1206,7 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             if edges is not None:
                 edgeTests = [e._toVHDL() for e in edges]
                 self.write("elsif ")
-                self.write("or ".join(edgeTests))
+                self.write(" or ".join(edgeTests))
                 self.write(" then")
             else:
                 self.write("else")
@@ -1253,12 +1266,18 @@ class _ConvertVisitor(ast.NodeVisitor, _ConversionMixin):
             s = n
             if isinstance(obj, bool):
                 s = "'%s'" % int(obj)
+                # print the symbol for a boolean in the global constant dict
+                if n in _constDict and obj == _constDict[n]:
+                    if isinstance(node.vhd, vhd_boolean):
+                        s = "bool(%s)" % n
             elif isinstance(obj, (int, long)):
-                # print the symbol for integer in the global constant dict
+                # print the symbol for an integer in the global constant dict
                 if n in _constDict and obj == _constDict[n]:
                     assert abs(obj) < 2**31
                     if isinstance(node.vhd, vhd_int):
                         s = n
+                    elif isinstance(node.vhd, vhd_boolean):
+                        s = "bool(%s)" % n
                     elif isinstance(node.vhd, vhd_std_logic):
                         s = "stdl(%s)" % n
                     elif isinstance(node.vhd, vhd_unsigned):
@@ -1784,6 +1803,7 @@ class _ConvertFunctionVisitor(_ConvertVisitor):
 
     def visit_Return(self, node):
         self.write("return ")
+        node.value.vhd = self.tree.vhd
         self.visit(node.value)
         self.write(";")
 
@@ -1870,7 +1890,7 @@ class vhd_unsigned(vhd_vector):
             ls = self.lenStr
             if ls:
                 return "unsigned(%s-1 downto 0)" % ls
-	    else:
+            else:
                 return "unsigned(%s downto 0)" % (self.size-1)
         else:
             return "unsigned"
@@ -1881,7 +1901,7 @@ class vhd_signed(vhd_vector):
             ls = self.lenStr
             if ls:
                 return "signed(%s-1 downto 0)" % ls
-	    else:
+            else:
                 return "signed(%s downto 0)" % (self.size-1)
         else:
             return "signed"
@@ -1994,6 +2014,8 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
             for a in node.args:
                 if isinstance(a, ast.Str):
                     a.vhd = vhd_unsigned(a.vhd.size)
+                elif isinstance(a.vhd, vhd_signed):
+                    a.vhd = vhd_unsigned(a.vhd.size) 
                 s += a.vhd.size
             node.vhd = vhd_unsigned(s)
         elif f is bool:
@@ -2078,10 +2100,14 @@ class _AnnotateTypesVisitor(ast.NodeVisitor, _ConversionMixin):
             left.vhd = vhd_unsigned(1)
         if isinstance(right.vhd, (vhd_boolean, vhd_std_logic)):
             right.vhd = vhd_unsigned(1)
-        if maybeNegative(left.vhd) and isinstance(right.vhd, vhd_unsigned):
-            right.vhd = vhd_signed(right.vhd.size + 1)
-        if isinstance(left.vhd, vhd_unsigned) and maybeNegative(right.vhd):
-            left.vhd = vhd_signed(left.vhd.size + 1)
+        if isinstance(right.vhd, vhd_unsigned):
+            if maybeNegative(left.vhd) or \
+               (isinstance(op, ast.Sub) and not hasattr(node, 'isRhs')): 
+                right.vhd = vhd_signed(right.vhd.size + 1)
+        if isinstance(left.vhd, vhd_unsigned): 
+            if maybeNegative(right.vhd) or \
+               (isinstance(op, ast.Sub) and not hasattr(node, 'isRhs')):
+                left.vhd = vhd_signed(left.vhd.size + 1)
         l, r = left.vhd, right.vhd
         ls, rs = l.size, r.size       
         if isinstance(r, vhd_vector) and isinstance(l, vhd_vector):
